/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.lookandfeel.impl;

import org.jdesktop.animation.timing.Animator;
import org.jdesktop.animation.timing.interpolation.PropertySetter;
import org.openide.windows.WindowManager;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class LightDimmerController 
  {
    private static LightDimmerController instance;
    
    private static final int ANIMATION_DURATION = 300;
    
    private static final float ANIMATION_DECELERATION = 0.8f;
    
    private Animator animator;
    
    private float intensity = 1f;
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private LightDimmerController()
      {
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public synchronized static LightDimmerController getInstance()
      {
        if (instance == null)
          {
            instance = new LightDimmerController();  
          }
        
        return instance;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public void setIntensity (final float intensity)
      {
        this.intensity = intensity;
        WindowManager.getDefault().getMainWindow().repaint();
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public void setIntensitySmoothly (final float intensity)
      {
        if ((animator != null) && animator.isRunning())
          {
            animator.stop();  
          }
        
        animator = PropertySetter.createAnimator(ANIMATION_DURATION, this, "intensity", this.intensity, intensity);
        animator.setDeceleration(ANIMATION_DECELERATION);
        animator.start();
      }
    
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public float getIntensity()
      {
        return intensity;    
      }
  }
